@@ -2,10 +2,10 @@
  *  cache-file.c
  *  mod_musicindex
  *
- *  $Id: cache-file.c 823 2009-02-05 19:38:14Z varenet $
+ *  $Id: cache-file.c 905 2010-01-08 13:16:43Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
- *  Copyright (c) 2003-2007 Thibaut VARENE
+ *  Copyright (c) 2003-2007,2009-2010 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -18,8 +18,8 @@
  * Flat file cache management subsystem.
  *
  * @author Thibaut Varene
- * @version $Revision: 823 $
- * @date 2003-2007
+ * @version $Revision: 905 $
+ * @date 2003-2009
  *
  * @warning 'errno' seems not to be seen as volatile by the compiler, therefore we
  *	cannot do "if(mkdir(foo) && errno == EEXIST)" for instance.
@@ -31,7 +31,6 @@
  */
 
 #include "playlist.h"
-#include "mod_musicindex.h"
 #include "cache-file.h"
 
 #ifdef HAVE_DIRENT_H
@@ -40,6 +39,7 @@
 #include <errno.h>
 #include <stdio.h>	/* fprintf / fscanf / fileno */
 #include <sys/file.h>	/* flock */
+#include <fcntl.h>	/* open */
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>	/* file handling */
 #endif
@@ -62,8 +62,6 @@
 #define CA_CREATE	30
 #define CA_LOCKED	40
 
-#define MAX_GENRE	200
-
 /** test for "." and ".." dir entries. x is a char * */
 #define ISDOT(x)	( (x)[0] == '.' && (!(x)[1] || ((x)[1] == '.' && !(x)[2])) )
 /** if char *x is NULL, print a printable ascii char (white space would be trashed by fscanf alas) */
@@ -83,10 +81,13 @@
  */
 static void error_handler(request_rec *r, const char *const caller)
 {
+	if (!r)
+		return;
+	
 	switch (errno) {
 		case EPERM:
 			/* The filesystem containing pathname does not support the creation of directories. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Can't create/delete directory.", caller);
+			mi_rerror("(%s) Can't create/delete directory.", caller);
 			break;
 #if 0
 		case EISDIR:
@@ -101,17 +102,17 @@ static void error_handler(request_rec *r, const char *const caller)
 		case EACCES:
 			/* The parent directory does not allow write permission to the  process,  or  one  of  the
 			directories in pathname did not allow search (execute) permission. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Permission denied.", caller);
+			mi_rerror("(%s) Permission denied.", caller);
 			break;
 		case EMFILE:
 			/* Too many file descriptors in use by process. */
 		case ENFILE:
 			/* Too many files are currently open in the system. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Too many open files!", caller);
+			mi_rerror("(%s) Too many open files!", caller);
 			break;
 		case ENAMETOOLONG:
 			/* pathname was too long. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Pathname was too long.", caller);
+			mi_rerror("(%s) Pathname was too long.", caller);
 			break;
 		case ENOENT:
 			/* A directory component in pathname does not exist or is a dangling symbolic link. */
@@ -124,27 +125,27 @@ static void error_handler(request_rec *r, const char *const caller)
 #endif
 		case ENOMEM:
 			/* Insufficient kernel memory was available. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Out Of Memory!", caller);
+			mi_rerror("(%s) Out Of Memory!", caller);
 			break;
 		case EROFS:
 			/* pathname refers to a file on a read-only filesystem. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Read-Only filesystem!", caller);
+			mi_rerror("(%s) Read-Only filesystem!", caller);
 			break;
 		case ELOOP:
 			/* Too many symbolic links were encountered in resolving pathname. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) Too many symbolic links.", caller);
+			mi_rerror("(%s) Too many symbolic links.", caller);
 			break;
 		case EIO:
 			/* An I/O error occured. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) I/O error.", caller);
+			mi_rerror("(%s) I/O error.", caller);
 			break;
 		case ENOSPC:
 			/* The device containing pathname has no room for the new directory.
 			The new directory cannot be created because the user's disk quota is exhausted. */
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) No space left on device!", caller);
+			mi_rerror("(%s) No space left on device!", caller);
 			break;
 		default:
-			ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) - error_handler! %s", caller, strerror(errno));
+			mi_rerror("(%s) - error_handler! %s", caller, strerror(errno));
 			break;
 	}
 	return;
@@ -161,15 +162,22 @@ static void error_handler(request_rec *r, const char *const caller)
  * @param r Apache request_rec struct to handle log writings.
  * @param dirpath A string representing a path to create.
  *
- * @return 0 on succes, CA_FATAL otherwise.
+ * @return CA_OK on succes, CA_FATAL otherwise.
  */
 static short file_cache_make_dir(request_rec *r, const char *const dirpath)
 {
 	register unsigned short l = 0, m = 0;
-	const char *tempdir = NULL;
+	char *tempdir = NULL;
+	short ret = CA_FATAL;
 
 	do {	/* We build the path subdirs by subdirs, in a "mkdir -p" fashion */
-		tempdir = apr_pstrndup(r->pool, dirpath, (m + (l = strcspn(dirpath + m, "/"))));
+		tempdir = realloc(tempdir, (m + (l = strcspn(dirpath + m, "/")) + 1));
+		if (!tempdir)
+			goto error_out;
+		
+		/* XXX TODO make better use of realloc() by using strncat() here */
+		strncpy(tempdir, dirpath, m + l);
+		tempdir[m+l] = '\0';
 		m += l;
 
 		if (!l)
@@ -185,11 +193,13 @@ static short file_cache_make_dir(request_rec *r, const char *const dirpath)
 		}
 	} while (1);
 
-	return 0;
-
+	ret = CA_OK;
+	
 error_out:
-	error_handler(r, __FUNCTION__);
-	return CA_FATAL;
+	free(tempdir);
+	if (ret == CA_FATAL)	// on est susceptible d'avoir de la merde en cas d'out of mem... pas grave
+		error_handler(r, __FUNCTION__);
+	return ret;
 }
 
 /**
@@ -250,21 +260,20 @@ static void file_cache_remove_dir(request_rec *r, DIR *cachedir, const char *con
  *
  * Basically we do nothing more here than creating the root cache folder.
  *
- * @param r Apache request_rec struct to handle log writings.
- * @param conf configuration structure in which the path to the cache root can be found.
+ * @param s Apache server_rec struct to handle log writings.
+ * @param cache_setup configuration string in which the path to the cache root can be found.
  *
  * @return CA_OK on succes, CA_FATAL otherwise.
  */
-static short file_cache_init(request_rec *r, const mu_config *const conf)
+static short file_cache_init(server_rec *s, const char *const cache_setup)
 {
-	chdir("/");	/* let's pray this will never fail */
-	if (file_cache_make_dir(r, (char *)(conf->cache_setup) + 1))	/* since we've chdir'd, we send the path without the leading '/' */
+	if (chdir("/") || file_cache_make_dir(NULL, cache_setup + 1))	/* since we've chdir'd, we send the path without the leading '/' */
 		goto error_out;
 
 	return CA_OK;
 
 error_out:
-	error_handler(r, __FUNCTION__);
+	mi_serror("Cannot setup file cache!");
 	return CA_FATAL;
 }
 
@@ -298,17 +307,8 @@ static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_co
 	if (!path)
 		return NULL;
 
-	/* Making sure the cache has been initialized, initialize it otherwise.
-	 * Bear in mind we're chdir'd from now on. */
-	if (chdir((char *)(conf->cache_setup))) { 			/* on va dans le rep de cache */
-		if (errno == ENOENT) {			/* il n'existe pas, il est temps d'initialiser le cache */
-			if (file_cache_init(r, conf))
-				return NULL;	/* l'init du cache a chie, c'est mauvais */
-			chdir((char *)(conf->cache_setup));
-		}
-		else
-			goto error_out;			/* un autre probleme, on degage */
-	}
+	/* Bear in mind we're chdir'd from now on. */
+	chdir((char *)(conf->cache_setup));
 
 	/* Actually check for the directory in the cache, create it if needed.
 	 * "+ 1" offset to suppress leading '/'. */
@@ -326,6 +326,8 @@ static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_co
 		if (cachedirstat.st_mtime < dirstat.st_mtime)	/* si la date de modif du rep de cache est plus vieille que celle du rep original, alors qqc a ete ajoute ou retire ou ecrit */
 			file_cache_remove_dir(r, cachedir, path);	/* alors on le vide proprement de son contenu */
 		closedir(cachedir);				/* On en a fini avec le repertoire, on le referme */
+		if (file_cache_make_dir(r, path + 1))	/* on recree le rep */
+			goto error_out;		
 	}
 
 	return NULL;
@@ -336,12 +338,12 @@ error_out:
 }
 
 /** struct type used to reduce memory usage */
-typedef struct {
+struct mi_data_buffer {
 	char title[MAX_STRING];
 	char album[MAX_STRING];
 	char artist[MAX_STRING];
 	char genre[MAX_GENRE];
-} mi_data_buffer;
+};
 
 /**
  * Fills in the information fields about a music file from the cache.
@@ -351,14 +353,14 @@ typedef struct {
  *
  * @warning @b MUST close in file handler on success.
  *
+ * @param r Apache request_rec struct to handle log writings.
  * @param pool Apache pool
  * @param in Not used (except for closing).
  * @param conf MusicIndex configuration paramaters struct.
  * @param names struct names to get the current filename.
- * @param r Apache request_rec struct to handle log writings.
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed,
- * 		Head pointer otherwise.
+ * 		NULL otherwise.
  */
 static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	const mu_config *const conf, mu_ent_names *const names)
@@ -367,46 +369,44 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 	short 		result = 0;
 	unsigned short	track, posn, flags, cvers = 0;
 	signed short	filetype;
+	int		fdesc;
 	FILE		*cache_file = NULL;
-	mi_data_buffer	*data_buffer = NULL;
+	struct mi_data_buffer	*data_buffer = NULL;
 
-	if (!conf->cache_setup)
-		return p;
-
-	/* Making sure the cache has been initialized, initialize it otherwise.
-	 * Bear in mind we're chdir'd from now on. */
-	if (chdir((char *)(conf->cache_setup))) {			/* on va dans le rep de cache */
-		if (file_cache_init(r, conf))
-			goto error_out;			/* l'init du cache a chie, c'est mauvais */
-		chdir((char *)(conf->cache_setup));
-	}
+	/* Bear in mind we're chdir'd from now on. */
+	chdir((char *)(conf->cache_setup));
 
 	/* Actually check for the file in the cache, open it if possible.
-	 * "+ 1" offset to suppress leading '/'. */
-	cache_file = fopen(names->filename + 1, "r");
-	if (!cache_file) {			/* on essaye d'ouvrir le fichier concerne en ro */
-		if (errno == ENOENT) {		/* n'existe pas encore mais on peut a priori le creer */
-			return p;		/* Creation of the file is handled separately (playlist.c) */
-		}
+	 * "+ 1" offset to suppress leading '/'.
+	 * Dev note: O_SHLOCK is BSD specific */
+	fdesc = open(names->filename + 1, O_RDONLY|O_NONBLOCK);
+	if (unlikely(fdesc < 0)) {
+		if (likely((errno == ENOENT) || (errno == EWOULDBLOCK) || (errno == EAGAIN)))
+			return p;	/* Creation of the file is handled separately (playlist.c) */
 		else
-			goto error_out;		/* game over */
+			goto error_out;	/* game over */
 	}
-
+	
 	/* We acquire a shared advisory lock on the file to be (almost) certain of its integrity.
 	 * This will prevent reading from incomplete cache files. The lock in non blocking:
 	 * if we can't get it, we won't wait to read the file, we'll delegate to the original handler. */
-	if (flock(fileno(cache_file), LOCK_SH|LOCK_NB)) {
-		fclose(cache_file);
+	if (flock(fdesc, LOCK_SH|LOCK_NB)) {
+		close(fdesc);
 		return p;
 	}
+	
+	cache_file = fdopen(fdesc, "r");
+	if (unlikely(!cache_file))
+		goto error_out;
+
 	/* Dev note: mixing unix and std IO is ugly, but there's no flockfile() counterpart to the shared
 	 * advisory lock, alas, see flockfile(3), flock(2) and lockf(3). Besides, we have to lock, since
 	 * fread()/fwrite() are thread safe, but not fscanf() and fprintf() */
 
-	p = NEW_ENT(r->pool);
-	if (p) {
-		data_buffer = (mi_data_buffer *)malloc(sizeof(mi_data_buffer)); /* This should save some memory */
-		if (data_buffer) {
+	p = NEW_ENT(pool);
+	if (likely(p)) {
+		data_buffer = (struct mi_data_buffer *)malloc(sizeof(struct mi_data_buffer)); /* This should save some memory */
+		if (likely(data_buffer)) {
 			result = fscanf(cache_file, "album: %[^\n]\nartist: %[^\n]\n"
 				"title: %[^\n]\ndate: %hu\ntrack: %hu\nposn: %hu\n"
 				"length: %hu\nbitrate: %lu\nfreq: %hu\nsize: %lu\n"
@@ -416,17 +416,17 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 				&p->bitrate, &p->freq, &p->size, &filetype, data_buffer->genre, &p->mtime, &flags, &cvers);
 		
 			/* Check whether the cache is somehow corrupted */
-			if ((result != CACHE_NF) || (cvers != CACHE_VERS)) {	/* fscanf() returns the number of input items assigned */
+			if (unlikely((result != CACHE_NF) || (cvers != CACHE_VERS))) {	/* fscanf() returns the number of input items assigned */
 				p = NULL;					/* hopefuly p allocs should be cleaned by apache */
 			}
 			else {
-				p->title = apr_pstrdup(r->pool, data_buffer->title);
+				p->title = apr_pstrdup(pool, data_buffer->title);
 				if (!ISBLANK(data_buffer->album))
-					p->album = apr_pstrdup(r->pool, data_buffer->album);
+					p->album = apr_pstrdup(pool, data_buffer->album);
 				if (!ISBLANK(data_buffer->artist))
-					p->artist = apr_pstrdup(r->pool, data_buffer->artist);
+					p->artist = apr_pstrdup(pool, data_buffer->artist);
 				if (!ISBLANK(data_buffer->genre))
-					p->genre = apr_pstrdup(r->pool, data_buffer->genre);
+					p->genre = apr_pstrdup(pool, data_buffer->genre);
 
 				/* We have to use that trick, fscanf won't work on char variables */
 				p->filetype = filetype;
@@ -441,11 +441,10 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 			p = NULL;	/* something failed, return non-bogus data. p will be cleaned up by apache's GC */
 	}
 
-	/* Explicitely releasing the lock, and closing the file */
-	flock(fileno(cache_file), LOCK_UN);
+	/* fclose() will also close() fdesc and thus release the lock */
 	fclose(cache_file);
 
-	if (p)
+	if (likely(p))
 		fclose(in);	/* this part of the cache subsystem is (uglily) seen as part of the playlist system,
 				and has to behave as such. This is why we close the input file if we took advantage of it. */
 
@@ -453,6 +452,7 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 
 error_out:
 	error_handler(r, __FUNCTION__);
+	close(fdesc);
 	return p;
 }
 
@@ -467,28 +467,39 @@ error_out:
  * @param conf The config structure used to find out cache configuration.
  * @param names A names structure in which the file name can be found.
  */
-static void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const conf, const mu_ent_names * const names)
+static void cache_file_write(request_rec *r, const mu_ent *const p, const mu_config *const conf, const mu_ent_names * const names)
 {
+	int	fdesc;
 	FILE	*cache_file = NULL;
 
+	/* we don't deal with directories themselves */
+	if (p->filetype < 0)
+		return;
+	
 	chdir((char *)conf->cache_setup);	/* Par securite on se re-chdir(). XXX Considere sans echec */
 	
-	cache_file = fopen(names->filename + 1, "w"); /* on ouvre le fichier en ecriture et en "truncate" (pour eviter les fichiers pourris) */
+	/* Dev note: O_EXLOCK is BSD specific. */
+	fdesc = open(names->filename + 1, O_WRONLY|O_NONBLOCK|O_CREAT, S_IRUSR|S_IWUSR);
+	if (fdesc < 0) {
+		if ((errno == EWOULDBLOCK) || (errno == EAGAIN))	/* does this work? */
+			return;	/* Creation of the file is handled separately (playlist.c) */
+		else
+			goto error_out;	/* game over */
+	}
+	
+	/* We acquire an exclusive advisory lock on the file to avoid corruption by another process.
+	 * This will also prevent reading from incomplete cache. see cache_read_file() comments. */
+	if (flock(fdesc, LOCK_EX|LOCK_NB)) {
+		fclose(cache_file);
+		return;
+	}
+	
+	cache_file = fdopen(fdesc, "w"); /* now open the fdesc with stdio routines */
 
 	/* let's check if something bad happened */
 	if (!cache_file)
 		goto error_out;
 
-	/* We acquire an exclusive advisory lock on the file to avoid corruption by another process.
-	 * This will also prevent reading from incomplete cache. see cache_read_file() comments. */
-	if (flock(fileno(cache_file), LOCK_EX|LOCK_NB)) {
-		fclose(cache_file);
-		if (errno == EWOULDBLOCK)
-			return;
-		else
-			goto error_out;
-	}
-
 	fprintf(cache_file, "album: %s\nartist: %s\ntitle: %s\ndate: %hu\n"
 		"track: %hhu\nposn: %hhu\nlength: %hu\nbitrate: %lu\nfreq: %hu\n"
 		"size: %lu\nfiletype: %hi\ngenre: %s\nmtime: %lu\nflags: %hhx\ncvers: %hu\n",
@@ -496,14 +507,14 @@ static void cache_file_write(request_rec *r, mu_ent *p, const mu_config *const c
 		p->track, p->posn, p->length, p->bitrate, p->freq, p->size, p->filetype,
 		BLANKSTR(p->genre), p->mtime, (p->flags & EF_FLAGSTOSAVE), CACHE_VERS);
 
-	/* Explicitely releasing the lock, and closing the file */
-	flock(fileno(cache_file), LOCK_UN);
+	/* fclose() will also close() fdesc and thus release the lock */
 	fclose(cache_file);
 
 	return;
 
 error_out:
 	error_handler(r, __FUNCTION__);
+	close(fdesc);
 }
 
 static const cache_backend cache_backend_file = {
@@ -516,13 +527,20 @@ static const cache_backend cache_backend_file = {
 
 int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *const conf)
 {
+	server_rec *s = cmd->server;
 	static const char biniou[] = "file://";
 
 	if (strncmp(biniou, setup_string, 7) == 0) {
-		conf->cache = &cache_backend_file;
 		conf->cache_setup = apr_pstrdup(cmd->pool, setup_string+6);
-		return 0;
+		if (!conf->cache_setup)
+			goto exit;
+		if (chdir((char *)(conf->cache_setup))) {
+			mi_serror("%s", strerror(errno));
+			goto exit;
+		}
+		conf->cache = &cache_backend_file;
+		return file_cache_init(s, conf->cache_setup);
 	}
-
+exit:
 	return 1;
 }
