@@ -2,11 +2,11 @@
  *  html.c
  *  mod_musicindex
  *
- *  $Id: html.c 826 2009-02-13 15:36:48Z varenet $
+ *  $Id: html.c 900 2009-12-06 21:48:19Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2008 Thibaut VARENE
+ *  Copyright (c) 2003-2009 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,8 +20,8 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 826 $
- * @date 2003-2008
+ * @version $Revision: 900 $
+ * @date 2003-2009
  *
  * That file groups all functions related to HTML and assimilated
  * generation, that is to say functions that are used to
@@ -72,7 +72,7 @@ static const char *const Gcovericns[] = {
  * Formats the list of songs to be sent.
  *
  * This function formats a list of tracks (if any) to be sent
- * to the client, as found in the @a p list passed in argument.
+ * to the client, as found in the @a pack passed in argument.
  * Depending on config options, it allows streaming/shuffle etc.
  *
  * @todo Sort option for recursive search?
@@ -81,7 +81,7 @@ static const char *const Gcovericns[] = {
  *	which also needs to be rewritten and simplified
  *
  * @param r Apache request_rec struct to handle connection details.
- * @param p struct mu_ent.
+ * @param pack pack of songs to display.
  * @param conf MusicIndex configuration paramaters struct.
  * @param customlist Whether the list of entries p is a custom list or not
  *
@@ -93,8 +93,8 @@ list_songs(request_rec *r, const mu_pack *const pack, const mu_config *const con
 {
 	const mu_ent *q;
 	register unsigned short fld = 0, j = 0;
-	const char *current = NULL, *new = NULL;
-	char *codeduri = NULL;
+	const char * restrict current = NULL, * restrict new = NULL;
+	char * restrict codeduri = NULL;
 	unsigned long tlength = 0, tsize = pack->fsize;
 
 	ap_rputs("  <tr class=\"title\">\n", r);
@@ -353,7 +353,8 @@ list_songs(request_rec *r, const mu_pack *const pack, const mu_config *const con
  * http://[user:][passwd\@]hostname[:port]/uri/
  *
  * @bug If the admin sets a conf->iceserver too big (>256) we're gonna be in
- *	trouble. Same thing goes if hostname+auth+etc is too long...
+ * trouble. Same thing goes if hostname+auth+etc is too long... Since there's
+ * no user-modifiable data here, we assume admins know what they're doing. 
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param uri A string (basically a path) to be sent to form a complete url.
@@ -361,12 +362,12 @@ list_songs(request_rec *r, const mu_pack *const pack, const mu_config *const con
  * @param conf MusicIndex configuration paramaters struct.
  * @param html 1 ? html output : normally escaped URL output.
  */
-static void send_url(request_rec *r, const char *const uri, const char *const command,
+static void send_url(request_rec *r, const char *const restrict uri, const char *const restrict command,
 			const mu_config *const conf, const short html)
 {
 	static char prefix[MAX_PREFIX];
 	static char str_port[7], strtmp[6];	/* (:)65536 + '\0' */
-	char *bufcoded, *decoded;
+	char * restrict bufcoded, * restrict decoded;
 	unsigned short l;
 	unsigned register i=0, j=6;
 
@@ -392,9 +393,10 @@ static void send_url(request_rec *r, const char *const uri, const char *const co
 			if (auth) {
 				bufcoded = strrchr(auth, ' ');
 				bufcoded++;
-				decoded = (char *)apr_palloc(r->pool, 1 + apr_base64_decode_len(bufcoded));
+				decoded = (char *)malloc(1 + apr_base64_decode_len(bufcoded));
 				l = apr_base64_decode(decoded, bufcoded);
 				strncat(prefix, decoded, l);	/* we have "user:pass" */
+				free(decoded);
 			}
 			strcat(prefix, "@");
 		}
@@ -453,7 +455,7 @@ void send_head(request_rec *r, const mu_config *const conf)
 		"\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
 		"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n"
 		"<head>\n"
-		" <meta name=\"generator\" content=\"mod_musicindex/" MUSIC_VERSION_STRING "\" />\n", r);
+		" <meta name=\"generator\" content=\"" MUSIC_HEADER_STRING "\" />\n", r);
 
 	/* XXX REVISIT: restoring a feature with probably not too shabby code? */
 	/* dive into user's customisations, if any */
@@ -792,7 +794,7 @@ void send_tracks(request_rec *r, const mu_pack *const pack, const mu_config *con
  * Sends track listing for the current custom playlist.
  *
  * This function takes care of preparing and sending to the client the
- * list (if any) of the available songs in the current custom p list,
+ * list (if any) of the available songs in the current custom @a pack list,
  * retrieved from a cookie.
  * Depending on config options, it allows streaming/shuffle etc.
  *
@@ -829,8 +831,11 @@ void send_customlist(request_rec *r, const mu_pack *const pack, const mu_config
 		_("Clear Playlist"), "</button>\n"
 		"   <button type=\"submit\" name=\"action\" value=\"StreamPlaylist\" class=\"playlist\">",
 		_("Stream Playlist"), "</button>\n"
+#ifdef ENABLE_OUTPUT_ARCHIVE
 		"   <button type=\"submit\" name=\"action\" value=\"DownloadPlaylist\" class=\"playlist\">",
-		_("Download Playlist"), "</button>\n", NULL);
+		_("Download Playlist"), "</button>\n"
+#endif
+		, NULL);
 
 	ap_rputs("  </div>\n"
 		" </form>\n"
@@ -916,7 +921,7 @@ void send_rss(request_rec *r, const mu_pack *const pack, const mu_config *const
 		ap_rputs("</itunes:summary>\n", r);
 	}
 
-	ap_rputs("  <generator>mod_musicindex/" MUSIC_VERSION_STRING "</generator>\n"
+	ap_rputs("  <generator>" MUSIC_HEADER_STRING "</generator>\n"
 		"  <docs>http://backend.userland.com/rss</docs>\n"
 		"  <ttl>60</ttl>\n",
 		r);
@@ -1068,16 +1073,20 @@ void send_randomdir(request_rec *r, const mu_config *const conf)
  *
  * This function takes care of the formating of the HTML page footers.
  * It sends the data that is common to all pages generated by the handler.
- * Currently it relies on apache_mp3 original stylesheets.
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param conf MusicIndex configuration paramaters struct.
+ * @param tvbegin when available, a timeval struct containing timing info for
+ *	the beginning of the page generation.
  */
-void send_foot(request_rec *r, const mu_config *const conf)
+void send_foot(request_rec *r, const mu_config *const conf, const STRUCTTV *tvbegin)
 {
 	char *icn = NULL;
 	request_rec *subreq = ap_sub_req_lookup_uri(conf->directory, r, NULL);
-
+#ifdef HAVE_GETTIMEOFDAY
+	struct timeval tvend, tvres;
+#endif
+	
 	ap_rputs("<!-- begin footer -->\n"
 		"<!-- mod_musicindex v." MUSIC_VERSION_STRING " -->\n"
 		"<!-- Authors: " MUSIC_AUTHORS_STRING " -->\n"
@@ -1114,9 +1123,19 @@ void send_foot(request_rec *r, const mu_config *const conf)
 			 "   <img src=\"http://cyber.law.harvard.edu/blogs/gems/tech/loveRss.gif\"\n"
 			 "    width=\"65\" height=\"59\" alt=\"Valid RSS feed.\" />\n"
 			 "  </a>\n", r);
+	ap_rputs(" </div>\n", r);
+
+#ifdef HAVE_GETTIMEOFDAY
+	/* timing */
+	gettimeofday(&tvend, NULL);
+	timersub(&tvend, tvbegin, &tvres);
+	ap_rputs(" <div id=\"timing\">", r);
+	ap_rprintf(r, _("in %u.%.6u s"), tvres.tv_sec, tvres.tv_usec);
+	ap_rputs("</div>\n", r);
+#endif
+
 	/* Signature */
-	ap_rputs(" </div>\n"
-		" <div id=\"name\">"
+	ap_rputs(" <div id=\"name\">"
 		"<a href=\"http://freshmeat.net/projects/musicindex/\">"
 		"MusicIndex v." MUSIC_VERSION_STRING "</a></div>\n"
 		"</div>\n"
