@@ -2,7 +2,7 @@
  *  html.h
  *  mod_musicindex
  *
- *  $Id: html.h 739 2007-04-22 22:25:05Z varenet $
+ *  $Id: html.h 878 2009-09-10 23:27:43Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -28,6 +28,6 @@ void send_customlist(request_rec *, const mu_pack *const, const mu_config *const
 void send_playlist(request_rec *, const mu_pack *const, const mu_config *const);
 void send_rss(request_rec *, const mu_pack *const, const mu_config *const);
 void send_randomdir(request_rec *, const mu_config *const);
-void send_foot(request_rec *, const mu_config *const);
+void send_foot(request_rec *, const mu_config *const, const STRUCTTV *);
 	
 #endif	/* HTML_H */
