@@ -2,7 +2,7 @@
  *  mod_musicindex.c
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.c 823 2009-02-05 19:38:14Z varenet $
+ *  $Id: mod_musicindex.c 900 2009-12-06 21:48:19Z varenet $
  *
  *  Created by Regis BOUDIN on Sat Dec 28 2002.
  *  Copyright (c) 2002-2006 Regis BOUDIN
@@ -49,7 +49,7 @@
  *	- <a href="ftp://ftp.mars.org/pub/mpeg/">libmad0</a>.
  *	- <a href="ftp://ftp.mars.org/pub/mpeg/">libid3tag0</a>.
  *	- <a href="http://flac.sourceforge.net/">libflac</a>.
- *	- <a href="http://mpeg4ip.sourceforge.net/">libmp4v2</a>.
+ *	- <a href="http://code.google.com/p/mp4v2/">libmp4v2</a>.
  *	- <a href="http://people.freebsd.org/~kientzle/libarchive/">libarchive</a>.
  *
  * @subpage devnotes
@@ -98,6 +98,27 @@
  *	- Finally, special care should be bore to storage space: always declare
  *	as @e little as possible: we want the module to consume as few
  *	memory as possible. That also applies to functions return types.
+ *
+ * @subsection restrictmod The restrict contract
+ *	I, [insert your name], a PROFESSIONAL or AMATEUR [circle one] programmer recognize
+ *	that there are limits to what a compiler can do. I certify that, to the best
+ *	of my knowledge, there are no magic elves or monkeys in the compiler which
+ *	through the forces of fairy dust can always make code faster. I understand
+ *	that there are some problems for which there is not enough information to
+ *	solve. I hereby declare that given the opportunity to provide the compiler
+ *	with sufficient information, perhaps through some key word, I will gladly use
+ *	said keyword and not bitch and moan about how "the compiler should be doing
+ *	this for me."
+ *
+ *	In this case, I promise that the pointer declared along with the restrict
+ *	qualifier is not aliased. I certify that writes through this pointer will
+ *	not effect the values read through any other pointer available in the same
+ *	context which is also declared as restricted.
+ *
+ *	Your agreement to this contract is implied by use of the restrict keyword ;)
+ *
+ *	http://www.cellperformance.com/mike_acton/2006/05/demystifying_the_restrict_keyw.html
+ *	http://developers.sun.com/solaris/articles/cc_restrict.html
  */
  
 /**
@@ -107,7 +128,7 @@
  * This file is the core of the module. It contains Apache's mandatory stuff.
  *
  * @author Regis Boudin
- * @version $Revision: 823 $
+ * @version $Revision: 900 $
  * @date 2002-2008
  *
  * http://httpd.apache.org/docs-2.0/developer/API.html
@@ -125,12 +146,8 @@
  * @todo review the randomdir implementation.
  *
  * @todo redesign cache integration to speed it up:
- *	- cache init on module initialization
  *	- cache access init (eg: db connex) on conn request
  *	- cache access cleanup (eg: db close) on end conn request
- *	- full real files bypass when possible (eg dir listing). ready
- *	- full cache-dependent search system
- *	(this is mostly done. Need to work on SQL implementation)
  *
  * @todo Check how much overhead a DAAP implementation would bring. The
  *	protocol works on top of HTTP (where we are) and could make use of the
@@ -160,8 +177,9 @@
 #ifdef HAVE_SETLOCALE
 #include <locale.h>
 #endif
-
-#define MUSIC_HEADER_STRING "mod_musicindex/" MUSIC_VERSION_STRING
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 
 #ifdef BUILD_FOR_APACHE2
 /** mime types handled */
@@ -221,6 +239,12 @@ static int handle_musicindex(request_rec *r)
 		.fsize = 0,
 	};
 
+	STRUCTTV tvbegin;
+	
+#ifdef HAVE_GETTIMEOFDAY
+	gettimeofday(&tvbegin, NULL);
+#endif
+
 	/* if the the module is not active, or if request is not a GET or a POST,
 	   let the dir to another module */
 	if (!(conf->options & MI_ACTIVE))
@@ -236,9 +260,7 @@ static int handle_musicindex(request_rec *r)
 	/* before doing anything (checking args, calling subfunctions), check we
 	   can open the directory */
 	if (access(r->filename, R_OK|X_OK) != 0) {
-		ap_log_rerror(APLOG_MARK, APLOG_ERR, r,
-			MI_LOG_PREFIX "(%s) Can't open directory: %s",
-			__func__, r->filename);
+		mi_rerror("Can't open directory: %s", r->filename);
 		return HTTP_FORBIDDEN;
 	}
 
@@ -357,7 +379,7 @@ static int handle_musicindex(request_rec *r)
 			send_directories(r, &master_pack, conf);
 		send_tracks(r, &master_pack, conf);
 		send_customlist(r, &custom_pack, conf);
-		send_foot(r, conf);
+		send_foot(r, conf, &tvbegin);
 	}
 
 	return OK;
