@@ -2,7 +2,7 @@
  *  mod_musicindex.h
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.h 824 2009-02-05 19:41:31Z varenet $
+ *  $Id: mod_musicindex.h 907 2010-01-08 13:32:36Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
@@ -45,12 +45,14 @@
  */
 #define MUSIC_VERSION_STRING	PACKAGE_VERSION		/**< Version string */
 #define MUSIC_AUTHORS_STRING	"R. Boudin & T. Varene"	/**< Authors */
+#define MUSIC_HEADER_STRING	"mod_musicindex/" MUSIC_VERSION_STRING
 
 /*
  * Compile time configuration options
  */
 #undef NO_TITLE_STRIP	/* Disable pretty printing for untagged files */
 #undef NO_BR_NOMINAL	/* Disable display of nominal bitrate when available */
+#undef DEBUG		/* Disable debug output */
 
 /* these are not defined on BSD */
 #ifndef FALSE
@@ -70,6 +72,23 @@
  #define unlikely(x)	(x)
 #endif
 
+#ifdef HAVE_SYS_TIME_H
+ #define STRUCTTV struct timeval
+ #if (!defined(HAVE_TIMERSUB) && !defined(timersub))
+  #define timersub(tvp, uvp, vvp)					\
+	do {								\
+		(vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;		\
+		(vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;	\
+		if ((vvp)->tv_usec < 0) {				\
+			(vvp)->tv_sec--;				\
+			(vvp)->tv_usec += 1000000;			\
+		}							\
+	} while (0)
+ #endif
+#else
+ #define STRUCTTV int
+#endif
+
 /* a very common macro */
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
 
@@ -189,8 +208,10 @@ enum {
 /*** End of mu_ent flags ***/
 
 #define MAX_STRING	1024		/**< Common reference for maximum string size. */
+#define MAX_GENRE	64		/**< max length for genre string. longest seen so far: 37 */
+#define MAX_FNAME	255		/**< max length for filenames. according to http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits 255 bytes/chars seems a common limit */
 
-#define MI_LOG_PREFIX	"[musicindex] "	/**< String used to prefix log messages */
+#define MI_LOG_PREFIX	"[mod_musicindex] "	/**< String used to prefix log messages */
 
 #ifndef BUILD_FOR_APACHE2
  
@@ -247,6 +268,14 @@ enum {
   #undef APLOG_ERR
   #define APLOG_ERR LOG_ERR, 0		/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
  #endif
+ #ifdef APLOG_WARNING
+  #undef APLOG_WARNING
+  #define APLOG_WARNING LOG_WARNING, 0	/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
+ #endif
+ #ifdef APLOG_DEBUG
+  #undef APLOG_DEBUG
+  #define APLOG_DEBUG LOG_DEBUG, 0	/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
+ #endif
 
  #define REQUEST_USER(rec) (rec->user)
 
@@ -258,8 +287,8 @@ enum {
  * while uri is the path to the file on the webserver.
  */
 typedef struct mu_ent_names {
-	char filename[MAX_STRING];	/**< file path */
-	char uri[MAX_STRING];		/**< file URI */
+	char filename[MAX_STRING];	/**< file path without trailing '/' */
+	char uri[MAX_STRING];		/**< file URI with trailing '/' */
 } mu_ent_names;
 
 /** 
@@ -280,10 +309,10 @@ typedef struct mu_ent {
 	const char *file;	/**< file uri. Relative uri unless MI_CUSTOM. */
 	const char *filename;	/**< filename (as in uri). file absolute *path* if MI_TARBALL */
 	const char *uri;	/**< file URI (unique) */
-	const char *album;	/**< album (if any) */
-	const char *artist;	/**< artist (if any) */
-	const char *title;	/**< title (as shown on the webpages) */
-	const char *genre;	/**< genre (type of music - if any) */
+	const char * restrict album;	/**< album (if any) */
+	const char * restrict artist;	/**< artist (if any) */
+	const char * restrict title;	/**< title (as shown on the webpages) */
+	const char * restrict genre;	/**< genre (type of music - if any) */
 	unsigned short date;	/**< date (year actually) */
 } /* __attribute__((packed)) */ mu_ent;
 
@@ -314,7 +343,7 @@ typedef struct {
 	    cache. If return is !NULL, (cache_backend::readdir()) will be
 	    subsequently called on the provided return pointer.
 	    See musicindex_opendir() for details. */
-	void* (*opendir)(request_rec *r, mu_pack *pack, const mu_config *const conf,
+	void* (*opendir)(request_rec *r, mu_pack *const pack, const mu_config *const conf,
 			const mu_ent_names * const names, unsigned long soptions);
 
 	/** If the above call provided a dir handler, this function will be used.
@@ -336,7 +365,8 @@ typedef struct {
 		const mu_config *const conf, mu_ent_names *const names);
 
 	/** This call tells the cache backend to write metadata for a given file. */
-	void (*write)(request_rec *r, mu_ent *entry, const mu_config *const conf, const mu_ent_names * const names);
+	void (*write)(request_rec *r, const mu_ent *const entry, const mu_config *const conf,
+				  const mu_ent_names * const names);
 } cache_backend;
 
 /** Configuration handling structure. We use it to share configuration
@@ -351,20 +381,20 @@ struct mu_config {
 	unsigned char order[SB_MAX+1];	/**< Sort order. */
 	unsigned char fields[SB_MAX+1];	/**< Fields to display */
 
-	const char *title;		/**< title of the page */
+	const char * restrict title;	/**< title of the page */
 
-	const char *directory;		/**< directory where musicindex files are stored */
+	const char * restrict directory;	/**< directory where musicindex files are stored */
 
-	const char *css;		/**< path to the css */
+	const char * restrict css;	/**< path to the css */
 
-	char *search;			/**< string to search */
+	char * restrict search;		/**< string to search */
 
-	const char *iceserver;		/**< adress of an icecast server */
+	const char * restrict iceserver;	/**< adress of an icecast server */
 
-	const char *custom_list;
+	const char * restrict custom_list;
 
-	const cache_backend *cache;
-	void *cache_setup;
+	const cache_backend * restrict cache;
+	void * restrict cache_setup;
 
 	signed short dir_per_line;	/**< number of directories to display per line. If <0 disable "pretty folders" */
 };
@@ -378,6 +408,18 @@ struct mu_config {
  #define _(String)	String
 #endif	/* HAVE_GETTEXT */
 
+#define mi_rerror(format, ...) \
+	ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "(%s) " format, __func__, ## __VA_ARGS__)
+#define mi_serror(format, ...) \
+	ap_log_error(APLOG_MARK, APLOG_ERR, s, MI_LOG_PREFIX "(%s) " format, __func__, ## __VA_ARGS__)
+
+/* this could arguably use APLOG_DEBUG but in most configurations this is never logged. Use it later ;-) */
+#ifdef DEBUG
+ #define mi_rdebug(format, ...) \
+	ap_log_rerror(APLOG_MARK, APLOG_WARNING, r, MI_LOG_PREFIX "(%s) " format, __func__, ## __VA_ARGS__)
+#else
+ #define mi_rdebug(format, ...)	/* nothing */
+#endif
 
 extern module MODULE_VAR_EXPORT musicindex_module;
 
