@@ -2,7 +2,7 @@
  *  playlist-mp3.c
  *  mod_musicindex
  *
- *  $Id: playlist-mp3.c 831 2009-02-14 18:26:52Z varenet $
+ *  $Id: playlist-mp3.c 899 2009-12-04 13:55:29Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jan 22 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 831 $
+ * @version $Revision: 899 $
  * @date 2003-2009
  *
  * This file contains everything needed to produce music entries from
@@ -42,7 +42,6 @@
 #endif
 
 #define INPUT_BUFFER_SIZE       16*1024	/**< Input buffer size used by libmad subsystem. */
-#define GENRE_MAX               0x94	/**< Max number of digit coded genres. */
 #define MAD_AVERAGE_FRAMES	10	/**< Number of frames to read before doing average calculations for VBR files */
 #define DO_FAST_CALC		1	/**< Whether we should read the whole file and get accurate bitrate/length or not */
 #define XING_MAGIC1		( ('X' << 24) | ('i' << 16) | ('n' << 8) | 'g' )	/**< XING magic header */
@@ -238,7 +237,7 @@ static id3_utf8_t *utf8_id3tag_findframe(struct id3_tag *tag, const char *const
 /**
  * Fills in the information fields about MP3 data.
  *
- * This function reads the id3 tags (using libid3tag0) from the MP3 file
+ * This function reads the id3 tags (using libid3tag0) from the MP3 file @a in
  * and fills in the struct mu_ent fields accordingly.
  *
  * @param pool Pool
@@ -359,7 +358,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			madread_size = fread(madinput_buffer + remaining, 1, INPUT_BUFFER_SIZE - remaining, in);
 
 			if (madread_size <= 0) {
-				ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "DBG: maderror madread_size <= 0 on %s", names->filename);
+				mi_rdebug("maderror madread_size <= 0 on %s", names->filename);
 				break;
 				/* TODO traitement eventuel erreur lecture */
 			}
@@ -372,7 +371,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 						break;
 					if (!MAD_RECOVERABLE(madstream.error)) {
 						/* TODO traitement eventuel erreur recoverable */
-						ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "DBG: maderror unrecoverable header decode on %s", names->filename);
+						mi_rdebug("maderror unrecoverable header decode on %s", names->filename);
 						break;
 					}
 					if (madstream.error == MAD_ERROR_LOSTSYNC) {
@@ -400,7 +399,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 					if (mad_frame_decode(&madframe, &madstream) == -1) {
 						if (!MAD_RECOVERABLE(madstream.error)) {
 							/* TODO traitement eventuel erreur recoverable */
-							ap_log_rerror(APLOG_MARK, APLOG_ERR, r, MI_LOG_PREFIX "DBG: maderror unrecoverable frame decode on %s", names->filename);
+							mi_rdebug("maderror unrecoverable frame decode on %s", names->filename);
 							break;
 						}
 					}
